# script to replicate labelerrors functionality
# 	Ralph Haugerud
#	USGS, Seattle

import arcgisscripting, sys, time, os.path

versionString = 'version of 24 May 2009'

# set Is93 = False for earlier (9.2) versions of ArcGIS
Is93 = True

fds = 'GeologicMap'
points = 'MapUnitPoints'
polys = 'MapUnitPolys'
mapUnit = 'MapUnit'

badLabels = 'errors_multilabels'
badPolys = 'errors_multilabelPolys'
blankPolys = 'errors_unlabeledPolys'
xxPolys = 'xxxpolys'
xxLabels = 'xxxlabels'

default = ''

def usage():
	print """
  Usage:  prompt> ncgmp09_labelerrors.py <geodatabaseName>
	Where <geodatabaseName> can be either a personal geodatabase or a file 
	geodatabase, .mdb or .gdb. The filename extension must be included. 
	<GeodatabaseName> must be an NCGMP09-style database with feature 
	data set GeologicMap that contains feature classes MapUnitPolys and 
	MapUnitPoints. MapUnitPolys must have attribute MapUnit.

	Output is 3 new feature classes within GeologicMap. Any existing 
	feature classes with these names will be overwritten: """
	print '	  '+badPolys
	print '	  '+badLabels
	print '	  '+blankPolys
	print '	This code also writes (overwrites) and deletes feature classes '
	print '	'+xxPolys+' and '+xxLabels+'.'

def elapsedTime(lastTime):
	thisTime = time.time()
	print '    %.1f sec' %(thisTime - lastTime)
	return thisTime

def main(lastTime,dbfds):
	if dbfds.find('.gdb') > 0: 
		isMDB = False
	else:
		isMDB = True

	print '  setting OverWriteOutput = True'
	gp.OverWriteOutput = True

	print '  intersecting (IDENTITY) points and polys'
	gp.workspace = dbfds
	gp.identity_analysis(points, polys, xxLabels)
	lastTime = elapsedTime(lastTime)

	FIDpolys = 'FID_'+polys
	FIDpoints = 'FID_'+points
	FieldList = ''
	SortFields = FIDpolys+' A; '+FIDpoints+' A'
	excessPoints = []
	badPolyList = []
	print '  getting label point rows'
	rows = gp.searchCursor(xxLabels,default,default,FieldList,SortFields)
	lastPoly = -2
	row = rows.next()
	appendFlag = True
	print '  iterating through label points'
	while row:
		if row.GetValue(FIDpolys) == lastPoly and appendFlag:
			badPolyList.append(row.GetValue(FIDpolys))
			appendFlag = False
		else:
			lastPoly = row.GetValue(FIDpolys)
			appendFlag = True
		row = rows.next()
	print '  copying polys to xxPolys'
	gp.copy_management(polys,xxPolys)
	OidName = gp.Describe(xxPolys).OIDFieldName
	print '  adding field MultipleLabels to badPolys'
	gp.AddField_management(xxPolys,'MultipleLabels','TEXT',default,default,5)
	print '  adding field MultipleLabels to badLabels'
	gp.AddField_management	(xxLabels,'MultipleLabels','TEXT',default,default,5)
	print '  getting xxPolys rows'
	gp.workspace = dbfds
	rows = gp.UpdateCursor(xxPolys)
	print '  iterating through xxPolys'
	row = rows.Next()
	while row:
		if row.getvalue(OidName)in badPolyList:
			row.SetValue('MultipleLabels','YES')
			rows.UpdateRow(row)
		row = rows.Next()
	print '  getting xxLabels rows'
	gp.workspace = dbfds
	rows = gp.UpdateCursor(xxLabels)
	print '  iterating through xxLabels'
	row = rows.Next()
	while row:
		if row.getvalue(FIDpolys) in badPolyList:
			row.SetValue('MultipleLabels','YES')
			rows.UpdateRow(row)
		row = rows.Next()
	lastTime = elapsedTime(lastTime)
	
	if isMDB:
		query = "[MultipleLabels] = 'YES'"
	else:
		query = '"MultipleLabels" = ' +"'YES'"
	print '  selecting multi-label polys to feature class '+badPolys
	gp.select_analysis(xxPolys,badPolys,query)
	print '  selecting multiple labels to feature class '+badLabels
	gp.select_analysis(xxLabels,badLabels,query)
	print '  selecting unlabeled polys to feature class '+blankPolys
	if isMDB:
		query = "[" + mapUnit + "] = ''"
	else:
		query = '"' + mapUnit + '" = ' + "''"
	gp.select_analysis(polys,blankPolys,query)
	
	# cleanup
	print '  cleaning up'
	del row, rows
	gp.delete_management(xxPolys)
	gp.delete_management(xxLabels)
	
	lastTime = elapsedTime(lastTime); print
	

### START HERE ###
startTime = time.time()
lastTime = startTime
print '  '+versionString
print '  checking input'
if len(sys.argv) <> 2 or not os.path.exists(sys.argv[1]):
	usage()
else:
	try:
		print '  loading arcprocessor'
		if Is93:
			gp = arcgisscripting.create(9.3)
		else:
			gp = arcgisscripting.create()
		lastTime = elapsedTime(lastTime)
	except:
		print '  Unable to create geoprocessing object. Arc license not present??'
	else:
		dbfds = sys.argv[1]+'/'+fds
		print '  testing for schema lock'
		if gp.TestSchemaLock(dbfds):
			lastTime = elapsedTime(lastTime)
			main(lastTime, dbfds)
			lastTime = elapsedTime(startTime)
		else:
			lastTime = elapsedTime(lastTime)
			print '  '+sys.argv[1]+' is locked. Close ArcMap or ArcCatalog?'
